<?php
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

// Import library dependencies
jimport('joomla.event.plugin');
jimport('joomla.environment.request');

class plgSystemJxUBBridge extends JPlugin
{
	private $debug = false;
	private $db = null;
	private $myinfo = array();
	private $user = null;
	
   /**
    * Constructor
    *
    * For php4 compatability we must not use the __constructor as a constructor for
    * plugins because func_get_args ( void ) returns a copy of all passed arguments
    * NOT references.  This causes problems with cross-referencing necessary for the
    * observer design pattern.
    */
    function plgSystemJxUBBridge( &$subject )
    {
	    parent::__construct( $subject );
	
	    // load plugin parameters
	    $this->_plugin = JPluginHelper::getPlugin( 'system', 'jxubbridge' );
	    $this->_params = new JParameter( $this->_plugin->params );
	    
	    // Check if debug mode is turned on.
	    $this->debug = $this->_params->get('debug');
	    
	    // Setup a db connection to UBB.threads
	    $option['driver']   = 'mysql';
		$option['host']     = $this->_params->get('db_host');
		$option['user']     = $this->_params->get('db_user');
		$option['password'] = $this->_params->get('db_pass');
		$option['database'] = $this->_params->get('db_name');
		$option['prefix']   = $this->_params->get('db_prefix');

		$this->db = & JDatabase::getInstance( $option );
		
		// Read the UBB.threads cookies
		$myinfo = array();
		$myinfo['id']      = JRequest::getVar($this->_params->get('cookie_prefix')."ubbt_myid", '', 'cookie', 'int');
		$myinfo['key']     = JRequest::getVar($this->_params->get('cookie_prefix')."ubbt_key", '', 'cookie', 'alnum');
		$myinfo['session'] = JRequest::getVar($this->_params->get('cookie_prefix')."ubbt_mysess", '', 'cookie', 'alnum');
		$myinfo['pass']    = JRequest::getVar($this->_params->get('cookie_prefix')."ubbt_pass", '', 'cookie', 'alnum');
		$myinfo['dob']     = JRequest::getVar($this->_params->get('cookie_prefix')."ubbt_dob", '', 'cookie', 'alnum');
		$this->myinfo = $myinfo;
		
    }
    
    /**
     * Method to log debug information to a file.
     */
    private function log( $message )
    {
    	if ($this->debug) {
	    	error_log(date("Y-m-d H:i:s") . " - $message\n", 3, '/tmp/jxubbridge.txt');
    	}
    }
    
    /**
     * Get the last node of the pathway.
     *
     * @return string $node Name of last node (current page).
     */
    private function getPathwayNode()
    {
		// Find out which page we are on.
		$pathway =& $mainframe->getPathWay();
		$breadcrumbs = $pathway->getNamePathWay();
		$node = array_pop($breadcrumbs);
		if (!$node) $node = $mainframe->getCfg('sitename');
		return $node;
    }
    
    /**
     * Updating this visit in the UBB.threads Online table.
     */
    private function updateThreadsOnlineTable()
    {
		// Read some default values.
		$IP = getenv("REMOTE_ADDR");
		$Last = time();

		$What = $this->getPathwayNode();

		// Update the Who's Online table for UBB.threads on each Joomla page load.
		if ( (empty($user['USER_DISPLAY_NAME'])) || ($bypass) ) {
			// Anonymous user or guest
			$online_display_name = "-ANON-$IP";
			$user_id = 1;
			$online_user_type = 'a';
		} else {
			// Logged in user			
			$online_display_name = $user['USER_DISPLAY_NAME'];
			$user_id = $user['USER_ID'];
			$online_user_type = 'r';
		}
		$query = "
		REPLACE INTO #__ONLINE
		(USER_ID,ONLINE_DISPLAY_NAME,ONLINE_LAST_ACTIVITY,ONLINE_SCRIPT_NAME,ONLINE_BROWSING_FORUM,ONLINE_USER_TYPE,ONLINE_USER_IP)
		VALUES ( '$user_id' , '$online_display_name' , '$Last', '$What', '', '$online_user_type', '$IP' )
		";
		$this->db->setQuery($query);
		$this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
    
    }
    
    /**
     * Load information about a UBB.threads user.
     *
     * @return object User object.
     */
    private function loadThreadsUser( $user_id )
    {
    	$query = "select * from #__USERS where USER_ID = '$user_id'";
    	$this->db->setQuery($query);
    	$user = $this->db->loadObject();
		$this->log('Query: ' . $this->db->getQuery());
    	return $user;
    }
    
    /**
     * Checks if the user is banned in UBB.threads.
     */
    private function checkBanned()
    {
    	if ($this->user->USER_IS_BANNED) return true;
    	
    	$IP = getenv('REMOTE_ADDR');

		$query = "
			SELECT	COUNT( BANNED_HOST )
			FROM	#__BANNED_HOSTS
			WHERE	BANNED_HOST LIKE '$IP'
		";
    	$this->db->setQuery($query);
    	$banned_ip_count = $this->db->loadResult();
		$this->log('Query: ' . $this->db->getQuery());
		
		if ($banned_ip_count > 0) return true;
		
		return false;
    }
    
    /**
     * Resets all UBB.threads cookies and session variables.
     */
    private function resetThreadsCookies()
    {
		$cookie_prefix = $this->_params->get('cookie_prefix');
		$cookie_path = $this->_params->get('cookie_path');
		$cookie_domain = $this->_params->get('cookie_domain');
		setcookie($cookie_prefix . "ubbt_myid", "", time() - 86400, $cookie_path, $cookie_domain);
		setcookie($cookie_prefix . "ubbt_key", "", time() - 86400, $cookie_path, $cookie_domain);
		setcookie($cookie_prefix . "ubbt_mysess", "", time() - 86400, $cookie_path, $cookie_domain);
		$_SESSION['forumvisit'] = array();
		$_SESSION['topicread'] = array();
		$_SESSION['watch_lists'] = array();    
    }
    
    /**
     * Sets the user session cookies for UBB.threads.
     */
    private function setThreadsCookies($Uid, $pass, $newsessionid)
    {
		$autolog = md5("$Uid$pass");
		$cookie_prefix = $this->_params->get('cookie_prefix');
		$cookie_path = $this->_params->get('cookie_path');
		$cookie_domain = $this->_params->get('cookie_domain');
		if ($remember) {
			$cookie_lifetime = time()+$this->_params->get('cookie_lifetime');
		} else {
			$cookie_lifetime = 0;
		}
		setcookie($cookie_prefix . "ubbt_myid", "$Uid", $cookie_lifetime, $cookie_path, $cookie_domain);
		setcookie($cookie_prefix . "ubbt_key", "$autolog", $cookie_lifetime, $cookie_path, $cookie_domain);
		setcookie($cookie_prefix . "ubbt_mysess", "$newsessionid", 0, $cookie_path, $cookie_domain);
		setcookie($cookie_prefix . "ubbt_pass", "", time()-3600, $cookie_path, $cookie_domain);
		setcookie($cookie_prefix . "ubbt_dob", "", time()-3600, $cookie_path, $cookie_domain);
    }
    
    /**
     * Calculate how many unread PMs a user has.
     * 
     * @param int $Uid User id to calculate unread PMs for.
     */
    private function calcUnreadPMs($Uid)
    {
		// Calculate unread PM's
		$query = "
			select	count(*)
			from		#__PRIVATE_MESSAGE_USERS as t1,
							#__PRIVATE_MESSAGE_TOPICS as t2
			where		t1.TOPIC_ID = t2.TOPIC_ID
			and			t2.TOPIC_LAST_REPLY_TIME > t1.MESSAGE_LAST_READ
			and			t1.USER_ID = '$Uid'
		";

		$this->db->setQuery($query);
		$total_unread = $this->db->loadResult($sth);
		$this->log('Query: ' . $this->db->getQuery());

		$query = "
			update	#__USER_PROFILE
			set			USER_TOTAL_PM = '$total_unread'
			where		USER_ID = '$Uid'
		";
		$this->db->setQuery($query);
		$this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
    }
    
    /**
     * Update the last visit time for user.
     *
     * @param int $Uid User id
     */
    private function updateThreadsLastVisit($Uid)
    {
		$date = time();
		$query = "
			update	#__USER_DATA
			set			USER_LAST_VISIT_TIME = '$date'
			where		USER_ID = '$Uid'
		";
		$this->db->setQuery($query);
		$this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
    }
    
    /**
     * Create a new session id and update the user with it.
     *
     * @param int $Uid User id.
     * @param int $sessionid The session id to set for the user.
     */
    private function updateThreadsUserSessionId($Uid, $sessionid)
    {
		$query = "
			update	#__USERS
			set			USER_SESSION_ID = '$sessionid'
			where		USER_ID = '$Uid'
		";
		$this->db->setQuery($query);
		$this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
    }
    
    /**
     * Remove anonymous IP from the UBB.threads Online table.
     */
    private function removeThreadsAnonymousIp()
    {
		$ip = "-ANON-" . getenv('REMOTE_ADDR');
		$query = "
			DELETE FROM #__ONLINE
			WHERE  ONLINE_DISPLAY_NAME = '$ip'
		";
		$this->db->setQuery($query);
		$this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
    }
    
    /**
     * Deletes an entry from the Who's Online table based on given display name.
     *
     * @param string $display_name Display name to delete from Who's Online.
     */
    private function deleteThreadsUserFromOnline($display_name)
    {
		$query = "
		DELETE FROM #__ONLINE
		WHERE  ONLINE_DISPLAY_NAME = '$display_name'
		";
		$this->db->setQuery($query);
		$this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
    }
    /**
     * Authenticate the UBB.threads user set in cookies.
     */
    private function authenticateThreadsUser()
    {
		// Load information about the current threads user.
		if ($myinfo['id'] > 1) {
			$this->user = $this->loadThreadsUser($myinfo['id']);
		} else {
			return false;
		}
		
		// Check if this user is banned.
		if ($this->checkBanned()) {
			// Maybe we need to reset all Joomla and UBB.threads user cookies here.
			$this->user = null;
			$this->resetThreadsCookies();
			return false;
		}
		
		// Check if cookie key is correct.
		if( $myinfo['key'] != md5( $this->user->USER_ID . $this->user->USER_PASSWORD ) ) {
			return false;
		}

		// Seems to be a legit UBB.threads user.
		$Uid = $myinfo['id'];
		
		// Update the last visit time for this user.
		$this->updateThreadsLastVisit($Uid);

		// Calculate how many unread PMs this user has.
		$this->calcUnreadPMs($Uid);

		return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onLoginUser( $credentials, $remember )
    {
        global $mainframe;

        $this->log('onLoginUser');

		// Get user credentials.        
        $Username = $credentials['username'];


		$query = "
			select	t1.USER_DISPLAY_NAME, t3.USER_LAST_VISIT_TIME, t1.USER_PASSWORD,t1.USER_ID,t2.USER_TEMPORARY_PASSWORD,t1.USER_IS_APPROVED,t1.USER_IS_BANNED,t1.USER_IS_UNDERAGE
			from		#__USERS as t1,
							#__USER_PROFILE as t2,
							#__USER_DATA as t3
			where		t1.USER_LOGIN_NAME = '$Username'
			and			t1.USER_ID = t2.USER_ID
			and			t1.USER_ID = t3.USER_ID
		";
		$this->db->setQuery($query);
		$user = $this->db->loadRow();
		list($CheckUser,$laston,$pass,$Uid,$temppass,$approved,$isbanned,$coppauser) = $user;
		
		$this->log('Query: ' . $this->db->getQuery());
		$this->log(print_r($user, true));
		
		// We need a user id to continue.
		if (!$Uid) return false;
        
        // Update the session id for this user.
        srand((double)microtime()*1000000);
        $newsessionid = md5(rand(0,32767));
		$this->updateThreadsUserSessionId($Uid, $newsessionid);

		// Update the last visit time for this user.
		$this->updateThreadsLastVisit($Uid);

		// Set UBB.threads user cookies
		$this->setThreadsCookies($Uid, $pass, $newsessionid);

		// Remove user's anonymous IP from online table.
		$this->removeThreadsAnonymousIp();

		// Calculate how many unread PMs this user has.
		$this->calcUnreadPMs($Uid);
		
		// Load the user information.
		$this->user = $this->loadThreadsUser($Uid);

        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onLogoutUser( $credentials )
    {
        global $mainframe;

        $this->log('onLogoutUser');


		// Get the User id.
		$Uid = $this->myinfo['id'];

		$query = "
			select
			t1.USER_DISPLAY_NAME,t1.USER_PASSWORD,t1.USER_SESSION_ID, t1.USER_MEMBERSHIP_LEVEL, t2.USER_TOTAL_PM, t1.USER_ID, t1.USER_IS_BANNED,t1.USER_IS_UNDERAGE, t2.USER_STYLE,t2.USER_HIDE_LEFT_COLUMN,t2.USER_HIDE_RIGHT_COLUMN
			from	#__USERS AS t1,
						#__USER_PROFILE as t2
			where t1.USER_ID = '$Uid'
			and		t1.USER_ID = t2.USER_ID
		";
		$this->db->setQuery($query);
		$this->log('Query: ' . $this->db->getQuery());
		$user = $this->db->loadAssoc();
		
		// Remove the user from the online page
		$this->deleteThreadsUserFromOnline($user['USER_DISPLAY_NAME']);

		// Reset the user session id for the user.	
		$this->updateThreadsUserSessionId($Uid, 0);
	
		// Update the last visit of this user.
		$this->updateThreadsLastVisit($Uid);
	
		// Delete UBB.threads user session cookies.
		$this->resetThreadsCookies();
        
        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onBeforeDeleteUser()
    {
        global $mainframe;

        $this->log('onBeforeDeleteUser');

        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onAfterDeleteUser()
    {
        global $mainframe;

        $this->log('onAfterDeleteUser');

        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    * 
    * @todo read $user info, check $bypass variable
    */
    function onAfterInitialize()
    {
        global $mainframe;

        $this->log('onAfterInitialise');
        
        // Authenticate this UBB.threads user.
        if (!$this->authenticateThreadsUser()) {
        	header("Location: index.php?option=com_user&task=logout");
        	die();
        }

		// Update the UBB.threads Online table.
		$this->updateThreadsOnlineTable();

        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onBeforeStoreUser()
    {
        global $mainframe;
        
        $this->log('onBeforeStoreUser');

        // Check if username already exists in UBB.threads

        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onAfterStoreUser($U, $UnqiueID)
    {
        global $mainframe;

        $this->log('onAfterStoreUser');
        
/*
    [id] => 64
    [name] => Kalle Anka
    [username] => kalleanka
    [email] => gurkmannen@yahoo.com
    [password] => cf9e06b878e78b20287263293cac857c:dwLUwQ4r01vFuqNmA9xhf8qJQ78eyIk
L
    [password_clear] => anka
    [usertype] => 
    [block] => 1
    [sendEmail] => 0
    [gid] => 18
    [registerDate] => 2008-05-07 21:23:44
    [lastvisitDate] => 
    [activation] => a8140a045616959cd39967d5b8ced29d
    [params] => 

    [aid] => 0
    [guest] => 1
    [password2] => anka
    [task] => register_save
    [719dc76eade7cc90f360a2a5ea4d1401] => 1
*/

		$this->log(print_r($U, true));
		$this->log("Unique ID: '$UniqueID'");
		
		// Use 'name' as display name if it exists, otherwise use 'username'.
		$Displayname = trim($U['name']) ? $U['name'] : $U['username'];
		$password = md5($U['password_clear']);
		$date = strtotime($U['registerDate']);
		$ip = getenv('REMOTE_ADDR');
		
		// Check if users get auto-approved. TODO: Not finished
		// No event for user approve, need to decide how to approve users in UBB.threads.
		if ($U['activation']) {
			$approved = $U['activation'];
		} else {
			$approved = "yes";
		}

		// Get default user title for new members.
		$query = "
			SELECT USER_TITLE_NAME
			FROM #__USER_TITLES
			WHERE USER_TITLE_POST_COUNT = '0'
		";
		$this->db->setQuery($query);
		$Title = $this->db->loadResult();
		$this->log('Query: ' . $this->db->getQuery());

		
        // Create this user in UBB.threads.
		$query = "
			INSERT INTO #__USERS (USER_LOGIN_NAME,USER_DISPLAY_NAME,USER_PASSWORD,USER_MEMBERSHIP_LEVEL,USER_REGISTERED_ON,USER_REGISTRATION_EMAIL,USER_REGISTRATION_IP,USER_IS_APPROVED,USER_IS_UNDERAGE)
			VALUES ('{$U["username"]}','$Displayname','{$U["password"]}','User','$date','{$U["email"]}','$ip','$approved','0')
		";
		$this->db->setQuery($query);
		$user = $this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
	
		$query = "
			select last_insert_id()
		";
		$this->db->setQuery($query);
		$Uid = $this->db->loadResult();
		$this->log('Query: ' . $this->db->getQuery());
	
		$query = "
			insert into #__USER_PROFILE
			(USER_ID,USER_REAL_EMAIL,USER_TOTAL_POSTS,USER_TITLE,USER_TOTAL_PM)
			values
			('$Uid','{$U["email"]}',0,'$Title',0)
		";
		$this->db->setQuery($query);
		$user = $this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
	
		$query = "
			insert into #__USER_DATA
			(USER_ID,USER_LAST_VISIT_TIME)
			values
			('$Uid','$date')
		";
		$this->db->setQuery($query);
		$user = $this->db->query();
		$this->log('Query: ' . $this->db->getQuery());
	
		// TODO: Must read values from config somehow.
		foreach($config['DEFAULT_USER_GROUPS'] as $key => $group) {
			$query = "
			insert into #__USER_GROUPS
			values
			('$Uid','$group')
			";
			$this->db->setQuery($query);
			$user = $this->db->query();
			$this->log('Query: ' . $this->db->getQuery());
		}   

        return true;
    }

    /**
    * Plugin method with the same name as the event will be called automatically.
    */
    function onAfterDisplayContent()
    {
        global $mainframe;

        //$this->log('onAfterDisplayContent');

        //return true;
    }

}
